#include "httpd.h"
#include <stdio.h>
#include <stdlib.h>
#include "buf_io.h"

int buf_fill(struct buf_io *io);

struct buf_io *buf_init(struct request_rec *r) {
    struct buf_io *io;

    io = (struct buf_io *)ap_pcalloc(r->pool, sizeof(*io));
    if (io == 0)
        return 0;

    io->r = r;
    
    io->f = ap_pfopen(io->r->pool, io->r->filename, "r");
    if (io->f == 0)
        return 0;

    io->in = ap_pcalloc(io->r->pool, BUF_IO_SIZE);
    if (io->in == 0)
        return 0;
    
    io->out = ap_pcalloc(io->r->pool, BUF_IO_SIZE);
    if (io->out == 0)
        return 0;

    io->b_in = io->in;
    io->b_out = io->out;
    io->b_in_size = BUF_IO_SIZE;
    io->b_out_size = BUF_IO_SIZE;
    
    buf_fill(io);

    if (io->b_in_level <= 0) {
        ap_pfclose(io->r->pool, io->f);
        return 0;
    }

    return io;
}

int buf_fill(struct buf_io *io) {
    io->b_in_level = fread(io->in, 1, io->b_in_size, io->f);

    if (io->b_in_level <= 0)
	return 0;
    
    io->b_in = io->in;
    
    return -1;
}

int buf_dump(struct buf_io *io) {
    ap_rprintf(io->r, "%s", io->b_out);
    io->out = io->b_out;
    memset(io->out, 0, io->b_out_size);
    io->b_out_level = 0;
    return 0;
}

unsigned char buf_getbyte(struct buf_io *io) {
    if (io->b_in_level == 0)
        buf_fill(io);
    
    if (io->b_in_level <= 0)
        return 255;
    
    io->b_in_level--;
    io->in++;

    return *((io->in-1));
}

int buf_add(struct buf_io *io, const char *buf, int len) {
    if (io->b_out_size <= len) {
        fprintf(stderr, "To large string recieved\n");
        return -1;
    }
    if ((io->b_out_level + len) >= (io->b_out_size - 1)) {
        /*fprintf(stderr, "Buffer limit reached, emptying\n");*/
        buf_dump(io);
    }
    memcpy(io->out+io->b_out_level, buf, len);
    io->b_out_level = io->b_out_level + len;
    return 0;
}

void buf_destroy(struct buf_io *io) {
    ap_pfclose(io->r->pool, io->f);
}

int parse_loop(struct buf_io *io) {
    unsigned char inchar;
    const char *pointer_url;
    const char *pointer_title;

    pointer_url = ap_table_get(io->r->notes, "pointer-url");
    pointer_title = ap_table_get(io->r->notes, "pointer-title");

    while ((inchar = buf_getbyte(io)) != 255) {
        if (inchar != '#') {
            buf_add(io, &inchar, 1);
        } else {
            if ((inchar = buf_getbyte(io)) != 255) {
                if (inchar != '#') {
                    buf_add(io, "#", 1);
                    buf_add(io, &inchar, 1);
                } else {
                    if ((inchar = buf_getbyte(io)) != 255) {
                        /* okay we have a token and need to *
                         * find the symbol we're replacing */
                        switch (inchar) {
                        case 'U':
                            /* URL of redirection */
                            buf_add(io, pointer_url, strlen(pointer_url));
                            break;
                        case 'T': 
                            /* TITLE of redirect page */
                            if (pointer_title == NULL || strcmp(pointer_title, "") == 0) {
                                buf_add(io, "Unknown", strlen("Unknown"));
                            } else {
                                buf_add(io, pointer_title, strlen(pointer_title));
                            }
                            break;
                        default:
                            fprintf(stderr, "Unknown option %c in template\n", inchar);
                            buf_add(io, "##", 2);
                            buf_add(io, &inchar, 1);
                            break;
                        }
                    }
                }
            }
        }
    }
    buf_dump(io);
    return 0;
}

